'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.resetCache = resetCache;

exports.default = function (target) {
  let options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  options = _extends({}, options, {
    max: 500
  });

  cache = (0, _lruCache2.default)(options);

  if (target instanceof _sequelize2.default.Association) {
    shimAssociation(target);
  } else if (target instanceof _sequelize2.default.Model) {
    shimModel(target);
    (0, _lodash.values)(target.associations).forEach(shimAssociation);
  } else if (target instanceof _sequelize2.default) {
    shimModel(target.Model.prototype);
    shimBelongsTo(target.Association.BelongsTo.prototype);
    shimHasOne(target.Association.HasOne.prototype);
    shimHasMany(target.Association.HasMany.prototype);
  }
};

var _sequelize = require('sequelize');

var _sequelize2 = _interopRequireDefault(_sequelize);

var _shimmer = require('shimmer');

var _shimmer2 = _interopRequireDefault(_shimmer);

var _dataloader = require('dataloader');

var _dataloader2 = _interopRequireDefault(_dataloader);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _lodash = require('lodash');

var _lruCache = require('lru-cache');

var _lruCache2 = _interopRequireDefault(_lruCache);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function mapResult(attribute, keys, options, result) {
  // Convert an array of results to an object of attribute (primary / foreign / target key) -> array of matching rows
  result = (0, _lodash.groupBy)(result, (0, _lodash.property)(attribute));

  return keys.map(key => {
    if (key in result) {
      let value = result[key];

      return options.multiple ? value : value[0];
    }
    return options.multiple ? [] : null;
  });
}

function loaderForModel(model, attribute) {
  let options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  let cacheKey = model.name + attribute + JSON.stringify(options);

  if (!cache.has(cacheKey)) {
    if (options.limit) {
      cache.set(cacheKey, new _dataloader2.default(keys => {
        if (keys.length === 1) {

          return model.findAll({
            where: {
              [attribute]: keys[0]
            },
            limit: options.limit
          }).then(mapResult.bind(null, attribute, keys, options));
        }

        return model.findAll({
          groupedLimit: {
            limit: options.limit,
            on: attribute,
            values: keys
          }
        }).then(mapResult.bind(null, attribute, keys, options));
      }, {
        cache: false
      }));
    } else {
      cache.set(cacheKey, new _dataloader2.default(keys => {
        return model.findAll({
          where: {
            [attribute]: keys
          }
        }).then(mapResult.bind(null, attribute, keys, options));
      }, {
        cache: false
      }));
    }
  }

  return cache.get(cacheKey);
}

function shimModel(target) {
  _shimmer2.default.massWrap(target, ['findById', 'findByPrimary'], original => {
    return function batchedFindById(id) {
      let options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      if (options.transaction) {
        return original.apply(this, arguments);
      }
      return loaderForModel(this, this.primaryKeyAttribute).load(id);
    };
  });
}

function shimBelongsTo(target) {
  _shimmer2.default.wrap(target, 'get', original => {
    return function batchedGetBelongsTo(instance) {
      let options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      // targetKeyIsPrimary already handled by sequelize (maps to findById)
      if (this.targetKeyIsPrimary || Array.isArray(instance) || !(0, _lodash.isEmpty)(options.where) || options.include || options.transaction) {
        return original.apply(this, arguments);
      }

      let loader = loaderForModel(this.target, this.targetKey);
      return loader.load(instance.get(this.foreignKey));
    };
  });
}

function shimHasOne(target) {
  _shimmer2.default.wrap(target, 'get', original => {
    return function batchedGetHasOne(instance) {
      let options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      if (Array.isArray(instance) || !(0, _lodash.isEmpty)(options.where) || options.include || options.transaction) {
        return original.apply(this, arguments);
      }

      let loader = loaderForModel(this.target, this.foreignKey);
      return loader.load(instance.get(this.sourceKey));
    };
  });
}

function shimHasMany(target) {
  _shimmer2.default.wrap(target, 'get', original => {
    return function bathedGetHasMany(instances) {
      let options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      if (!(0, _lodash.isEmpty)(options.where) || options.include || options.transaction) {
        return original.apply(this, arguments);
      }

      let loader = loaderForModel(this.target, options.limit ? this.foreignKeyField : this.foreignKey, _extends({}, options, {
        multiple: true
      }));

      if (Array.isArray(instances)) {
        return _bluebird2.default.map(instances, instance => loader.load(instance.get(this.source.primaryKeyAttribute)));
      } else {
        return loader.load(instances.get(this.source.primaryKeyAttribute));
      }
    };
  });
}

function shimAssociation(association) {
  if (association instanceof _sequelize2.default.Association.BelongsTo) shimBelongsTo(association);else if (association instanceof _sequelize2.default.Association.HasOne) shimHasOne(association);else if (association instanceof _sequelize2.default.Association.HasMany) shimHasMany(association);
}

let cache;
function resetCache() {
  if (cache) cache.reset();
}